/*
 * Name:    mp_mca.c
 *
 * Purpose: Python module interface to generic MX MCA functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_mca.h"
#include "mx_handle.h"
#include "mx_array.h"
#include "mp.h"

MX_EXPORT PyObject *
MpPrivate_mca_is_busy_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status, busy;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_is_busy( record, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", busy );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_start_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_start( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_stop_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_stop( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_read_cmd( PyObject *self, PyObject *args )
{
	PyObject *channel_object, *channels_tuple;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long i, num_channels;
	unsigned long *channel_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_read( record, &num_channels, &channel_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to hold the returned channel values. */

	channels_tuple = PyTuple_New( (int) num_channels );

	if ( channels_tuple == NULL )
		return NULL;

	/* Insert the channel values into the tuple. */

	for ( i = 0; i < num_channels; i++ ) {

		channel_object = PyInt_FromLong( (long) ( channel_array[i] ) );

		if ( channel_object == NULL ) {
			Py_DECREF( channels_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( channels_tuple, i, channel_object );
	}

	return channels_tuple;
}

MX_EXPORT PyObject *
MpPrivate_mca_clear_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_clear( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_start_without_preset_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_start_without_preset( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_start_with_preset_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status, preset_type;
	double preset_value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lid", &handle,
						&preset_type, &preset_value );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_start_with_preset( record,
						preset_type, preset_value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_start_for_live_time_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double preset_live_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld",
						&handle, &preset_live_time );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_start_for_preset_live_time( record,
							preset_live_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_start_for_real_time_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double preset_real_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld",
						&handle, &preset_real_time );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_start_for_preset_real_time( record,
							preset_real_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_start_for_preset_count_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long preset_count;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
						&handle, &preset_count );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_start_for_preset_count( record,
						(unsigned long) preset_count );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_preset_type_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status, preset_type;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_preset_type( record, &preset_type );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", preset_type );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_set_preset_type_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status, preset_type;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "li",
					&handle, &preset_type );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_set_preset_type( record, preset_type );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_preset_count_region_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle;
	unsigned long preset_count_region[2];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_preset_count_region( record,
						preset_count_region );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The preset count region boundaries are returned
	 * as a 2-element tuple.
	 */

	result = Py_BuildValue( "ll", (long) preset_count_region[0],
					(long) preset_count_region[1] );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_set_preset_count_region_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "set_preset_count_region";

	PyObject *preset_count_region_tuple;
	MX_RECORD *record;
	int parse_status, tuple_size;
	long handle;
	unsigned long preset_count_region[2];
	long lower_boundary, upper_boundary;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lO",
				&handle, &preset_count_region_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The last argument is expected to be a 2-element tuple containing
	 * the region of interest boundaries.
	 */

	if ( PyTuple_Check( preset_count_region_tuple ) == 0 ) {
		MpPrivateErr_SetString( PyExc_TypeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but it is not a tuple.", cname );

		return NULL;
	}

	tuple_size = PyTuple_Size( preset_count_region_tuple );

	if ( tuple_size != 2 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but instead the tuple has %d elements.",
		cname, tuple_size );

		return NULL;
	}

	/* Copy the preset_count_region boundaries in the tuple
	 * to the preset_count_region array.
	 */

	parse_status = PyArg_ParseTuple( preset_count_region_tuple, "ll",
					&lower_boundary, &upper_boundary );

	if ( parse_status == 0 )
		return NULL;

	preset_count_region[0] = (unsigned long) lower_boundary;
	preset_count_region[1] = (unsigned long) upper_boundary;

	mx_status = mx_mca_set_preset_count_region( record,
						preset_count_region );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_roi_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, roi_number;
	unsigned long roi[2];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &roi_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_roi( record, (unsigned long) roi_number, roi );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The region of interest boundaries are returned
	 * as a 2-element tuple.
	 */

	result = Py_BuildValue( "ll", (long) roi[0], (long) roi[1] );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_set_roi_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "set_roi";

	PyObject *roi_tuple;
	MX_RECORD *record;
	int parse_status, tuple_size;
	long handle, roi_number;
	unsigned long roi[2];
	long lower_boundary, upper_boundary;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llO",
			&handle, &roi_number, &roi_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The last argument is expected to be a 2-element tuple containing
	 * the region of interest boundaries.
	 */

	if ( PyTuple_Check( roi_tuple ) == 0 ) {
		MpPrivateErr_SetString( PyExc_TypeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but it is not a tuple.", cname );

		return NULL;
	}

	tuple_size = PyTuple_Size( roi_tuple );

	if ( tuple_size != 2 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but instead the tuple has %d elements.",
		cname, tuple_size );

		return NULL;
	}

	/* Copy the roi boundaries in the tuple to the roi array. */

	parse_status = PyArg_ParseTuple( roi_tuple, "ll",
					&lower_boundary, &upper_boundary );

	if ( parse_status == 0 )
		return NULL;

	roi[0] = (unsigned long) lower_boundary;
	roi[1] = (unsigned long) upper_boundary;

	mx_status = mx_mca_set_roi( record, (unsigned long) roi_number, roi );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_roi_integral_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, roi_number;
	unsigned long roi_integral;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &roi_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_roi_integral( record,
						(unsigned long) roi_number,
						&roi_integral );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) roi_integral );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_rois_cmd( PyObject *self, PyObject *args )
{
	const char fname[] = "MpPrivate_mca_get_rois_cmd()";

	PyObject *rois_tuple, *boundaries_tuple;
	MX_RECORD *record;
	MX_MCA *mca;
	long handle;
	int parse_status;
	unsigned long i, current_num_rois;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Find out how many ROIs are currently in use. */

	mx_status = mx_mca_get_pointers( record, &mca, NULL, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	current_num_rois = mca->current_num_rois;

	/* Read the ROI boundaries for all ROIs. */

	mx_status = mx_mca_get_roi_array( record, current_num_rois, NULL );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Create a tuple to contain the set of ROI boundaries. */

	rois_tuple = PyTuple_New( (int) current_num_rois );

	if ( rois_tuple == NULL )
		return NULL;

	/* Copy the ROI boundaries to the new tuple. */

	for ( i = 0; i < current_num_rois; i++ ) {

		/* Each element of rois_tuple is itself a 2-element tuple
		 * containing the boundaries of one ROI.
		 */

		boundaries_tuple = Py_BuildValue( "ll",
						(mca->roi_array)[i][0],
						(mca->roi_array)[i][1] );

		if ( boundaries_tuple == NULL ) {
			Py_DECREF( rois_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( rois_tuple, i, boundaries_tuple );
	}

	return rois_tuple;
}

MX_EXPORT PyObject *
MpPrivate_mca_set_rois_cmd( PyObject *self, PyObject *args )
{
	const char fname[] = "MpPrivate_mca_set_rois_cmd()";
	const char cname[] = "set_rois";

	PyObject *rois_tuple, *boundaries_tuple;
	MX_RECORD *record;
	MX_MCA *mca;
	int parse_status;
	long handle;
	unsigned long i, current_num_rois;
	unsigned long **roi_array;
	long lower_boundary, upper_boundary;
	long dimension_array[2];
	size_t element_size_array[2];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lO", &handle, &rois_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Find out how many ROIs are currently in use. */

	mx_status = mx_mca_get_pointers( record, &mca, NULL, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	current_num_rois = mca->current_num_rois;

	/* The last argument 'rois_tuple' should be a tuple. */

	if ( PyTuple_Check( rois_tuple ) == 0 ) {
		MpPrivateErr_SetString( PyExc_TypeError,
		"The last argument to '%s' is expected to be "
		"a tuple, but it is not.", cname );

		return NULL;
	}

	dimension_array[0] = (long) current_num_rois;
	dimension_array[1] = 2;

	element_size_array[0] = sizeof( unsigned long );
	element_size_array[1] = sizeof( unsigned long * );

	roi_array = (unsigned long **) mx_allocate_array( 2,
					dimension_array, element_size_array );

	if ( roi_array == NULL ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
		"Ran out of memory trying to allocate a %lu by 2 element "
		"ROI array.", current_num_rois );

		return NULL;
	}

	/* Copy the ROI boundaries to roi_array. */

	for ( i = 0; i < current_num_rois; i++ ) {

		boundaries_tuple = PyTuple_GetItem( rois_tuple, (int) i );

		if ( boundaries_tuple == NULL ) {
			mx_free_array( roi_array, 2, dimension_array,
						element_size_array );
			return NULL;
		}

		parse_status = PyArg_ParseTuple( boundaries_tuple, "ll",
					&lower_boundary, &upper_boundary );

		if ( parse_status == 0 ) {
			mx_free_array( roi_array, 2, dimension_array,
						element_size_array );
			return NULL;
		}

		roi_array[i][0] = (unsigned long) lower_boundary;
		roi_array[i][1] = (unsigned long) upper_boundary;
	}

	/* Set all the ROI boundaries. */

	mx_status = mx_mca_set_roi_array( record, current_num_rois, roi_array );

	mx_free_array( roi_array, 2, dimension_array, element_size_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
	
	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_roi_integrals_cmd( PyObject *self, PyObject *args )
{
	const char fname[] = "MpPrivate_mca_get_roi_integrals_cmd()";

	PyObject *roi_integral_object, *roi_integrals_tuple;
	MX_RECORD *record;
	MX_MCA *mca;
	long handle;
	int parse_status;
	unsigned long i, current_num_rois;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Find out how many ROIs are currently in use. */

	mx_status = mx_mca_get_pointers( record, &mca, NULL, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	current_num_rois = mca->current_num_rois;

	/* Get the ROI integral values. */

	mx_status = mx_mca_get_roi_integral_array( record,
						current_num_rois, NULL );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to hold the returned ROI integral values. */

	roi_integrals_tuple = PyTuple_New( (int) current_num_rois );

	if ( roi_integrals_tuple == NULL )
		return NULL;

	/* Insert the ROI integral values into the tuple. */

	for ( i = 0; i < current_num_rois; i++ ) {

		roi_integral_object = PyFloat_FromDouble(
						(mca->roi_integral_array)[i] );

		if ( roi_integral_object == NULL ) {
			Py_DECREF( roi_integrals_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( roi_integrals_tuple, i, roi_integral_object );
	}

	return roi_integrals_tuple;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_num_channels_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long num_channels;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_num_channels( record, &num_channels );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) num_channels );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_set_num_channels_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long num_channels;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&handle, &num_channels );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_set_num_channels( record,
					(unsigned long) num_channels );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_channel_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, channel_number;
	unsigned long channel_value;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
						&handle, &channel_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_channel( record,
					(unsigned long) channel_number,
					&channel_value );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) channel_value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_real_time_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double real_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_real_time( record, &real_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", real_time );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_live_time_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double live_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_live_time( record, &live_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", live_time );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_counts_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long counts;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_counts( record, &counts );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) counts );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_soft_roi_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, soft_roi_number;
	unsigned long soft_roi[2];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
						&handle, &soft_roi_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_soft_roi( record,
			(unsigned long) soft_roi_number, soft_roi );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The region of interest boundaries are returned
	 * as a 2-element tuple.
	 */

	result = Py_BuildValue( "ll", (long) soft_roi[0], (long) soft_roi[1] );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_mca_set_soft_roi_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "set_soft_roi";

	PyObject *soft_roi_tuple;
	MX_RECORD *record;
	int parse_status, tuple_size;
	long handle, soft_roi_number;
	unsigned long soft_roi[2];
	long lower_boundary, upper_boundary;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llO",
			&handle, &soft_roi_number, &soft_roi_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* The last argument is expected to be a 2-element tuple containing
	 * the region of interest boundaries.
	 */

	if ( PyTuple_Check( soft_roi_tuple ) == 0 ) {
		MpPrivateErr_SetString( PyExc_TypeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but it is not a tuple.", cname );

		return NULL;
	}

	tuple_size = PyTuple_Size( soft_roi_tuple );

	if ( tuple_size != 2 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
		"The last argument to '%s' is expected to be "
		"a 2-element tuple, but instead the tuple has %d elements.",
		cname, tuple_size );

		return NULL;
	}

	/* Copy the soft_roi boundaries in the tuple to the soft_roi array. */

	/* Copy the roi boundaries in the tuple to the roi array. */

	parse_status = PyArg_ParseTuple( soft_roi_tuple, "ll",
					&lower_boundary, &upper_boundary );

	if ( parse_status == 0 )
		return NULL;

	soft_roi[0] = (unsigned long) lower_boundary;
	soft_roi[1] = (unsigned long) upper_boundary;

	mx_status = mx_mca_set_soft_roi( record,
				(unsigned long) soft_roi_number, soft_roi );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_mca_get_soft_roi_integral_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, soft_roi_number;
	unsigned long soft_roi_integral;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
						&handle, &soft_roi_number );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_mca_get_soft_roi_integral( record,
					(unsigned long) soft_roi_number,
					&soft_roi_integral );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", (long) soft_roi_integral );

	return result;
}

