/*
 * Name:    mp_util.c
 *
 * Purpose: This file contains the code that initializes the underlying
 *          MX library.  It also includes other miscellaneous functions
 *          that do not fit elsewhere.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include <signal.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_version.h"
#include "mx_handle.h"
#include "mx_plot.h"
#include "mp.h"

MX_EXPORT PyObject *
MpPrivate_init_cmd( PyObject *self, PyObject *args )
{
	const char fname[] = "MpPrivate_init_cmd()";

	mx_status_type mx_status;

	mx_set_debug_level(0);

	MX_DEBUG( 2,("%s invoked", fname));

#ifdef SIGPIPE
	/* Don't need sigaction() just to ignore a signal. */

	signal( SIGPIPE, SIG_IGN );
#endif

	/* Also trap ctrl-C interrupts. */

	signal( SIGINT, SIG_IGN );

	/* Initialize the subsecond sleep functions (if they need it). */

	mx_msleep(1);

	/* Initialize the MX time keeping functions. */

	mx_initialize_clock_ticks();

	/* Initialize the drivers for the various MX record types. */

	mx_status = mx_initialize_drivers();

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Create the global handle table for the MX records.  Configure the
	 * table to be incremented in blocks of 100 handles.
	 */

	mx_status = mx_create_handle_table(
					&MpPrivate_global_record_handle_table,
					100, 1 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_get_debug_level_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	int debug_level;

	debug_level = mx_get_debug_level();

	result = Py_BuildValue( "i", debug_level );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_set_debug_level_cmd( PyObject *self, PyObject *args )
{
	int debug_level, parse_status;

	parse_status = PyArg_ParseTuple( args, "i", &debug_level );

	if ( parse_status == 0 )
		return NULL;

	mx_set_debug_level( debug_level );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_plotting_is_enabled_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int plot_enable_value, parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	plot_enable_value = mx_plotting_is_enabled( record );

	result = Py_BuildValue( "i", plot_enable_value );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_set_plot_enable_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int plot_enable_value, parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "li",
					&handle, &plot_enable_value );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_set_plot_enable( record, plot_enable_value );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_set_program_name_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record_list;
	long handle;
	char *program_name;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ls", &handle, &program_name );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record_list );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_set_program_name( record_list, program_name );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

/* MpPrivateErr_SetString() is a varargs version of PyErr_SetString. */

MX_EXPORT void
MpPrivateErr_SetString( PyObject *exception_object, char *format, ... )
{
	va_list args;
	char buffer[2500];

	va_start(args, format);
	vsprintf(buffer, format, args);
	va_end(args);

	PyErr_SetString( exception_object, buffer );

	return;
}

/* MpPrivateErr_SetFromMxStatus() translates between MX errors
 * and Python exceptions.
 */

MX_EXPORT PyObject *
MpPrivateErr_SetFromMxStatus( PyObject *error_object, mx_status_type mx_status )
{
	if ( error_object == (PyObject *) NULL ) {

		switch( mx_status.code ) {
		case MXE_SUCCESS:
		case MXE_NULL_ARGUMENT:
		case MXE_ILLEGAL_ARGUMENT:
		case MXE_CORRUPT_DATA_STRUCTURE:
		case MXE_UNPARSEABLE_STRING:
		case MXE_END_OF_DATA:
		case MXE_UNEXPECTED_END_OF_DATA:
		case MXE_NOT_FOUND:
		case MXE_TYPE_MISMATCH:
		case MXE_NOT_YET_IMPLEMENTED:
		case MXE_UNSUPPORTED:
		case MXE_OUT_OF_MEMORY:
		case MXE_WOULD_EXCEED_LIMIT:
		case MXE_LIMIT_WAS_EXCEEDED:
		case MXE_INTERFACE_IO_ERROR:
		case MXE_DEVICE_IO_ERROR:
		case MXE_FILE_IO_ERROR:
		case MXE_TERMINAL_IO_ERROR:
		case MXE_IPC_IO_ERROR:
		case MXE_IPC_CONNECTION_LOST:
		case MXE_NETWORK_IO_ERROR:
		case MXE_NETWORK_CONNECTION_LOST:
		case MXE_NOT_READY:
		case MXE_INTERRUPTED:
		case MXE_PAUSE_REQUESTED:
		case MXE_INTERFACE_ACTION_FAILED:
		case MXE_DEVICE_ACTION_FAILED:
		case MXE_FUNCTION_FAILED:
		case MXE_CONTROLLER_INTERNAL_ERROR:
		case MXE_PERMISSION_DENIED:
		case MXE_CLIENT_REQUEST_DENIED:
		case MXE_TRY_AGAIN:
		case MXE_TIMED_OUT:
		case MXE_HARDWARE_CONFIGURATION_ERROR:
		case MXE_HARDWARE_FAULT:
		default:
			error_object = PyExc_Exception;
			break;
		}
	}

	PyErr_SetString( error_object, mx_status.message );

	return NULL;
}

